@@ -96,6 +96,18 @@ void ConfigElem::init( )
 
   //set default values
   PasswordStorage = DEFAULT_ACCOUNT_PASSWORD_STORAGE;
+  filterApplied = false;
+  deletionPerformedByFilters = false;
+  refreshPerformedByFilters = false;
+  downloadActionsInvoked = false;
+
+  //initialize counters
+  moveCounter = 0;
+  nmbDeletedMailsLastRefresh = 0;
+  nmbDeletedMailsLastStart = 0;
+  nmbMovedMailsLastRefresh = 0;
+  nmbMovedMailsLastStart = 0;
+  nmbIgnoredMails = 0;
 }
 
 
@@ -131,11 +143,6 @@ int ConfigElem::count()
   return m_pshowrecord->count();
 }
 
-void ConfigElem::applyFilters()
-{
-  m_pshowrecord->applyFilters();
-}
-
 bool ConfigElem::isActive( ) const
 {
   return m_bActive;
@@ -311,7 +318,7 @@ bool ConfigElem::assertPassword( bool force )
       config->sync();
 
       //emit configuration changed signal
-      emit sigConfigChanged();
+      emit ( sigConfigChanged() );
 
       //tell we have a password
       return true;
@@ -331,7 +338,14 @@ void ConfigElem::deleteNextMail( )
   //if the list of mails to delete is empty, finalize the deletion and return
   if( MailsToDelete.empty() )
   {
-    commitDeletion();
+    if( deletionPerformedByFilters )
+    {
+      applyFiltersDeleted();
+    }
+    else
+    {
+      commitDeletion();
+    }
     return;
   }
 
@@ -356,7 +370,14 @@ void ConfigElem::slotMailDeleted( KIO::Job* job )
     if( res == false )
     {
       //we have not got a new password; cancel delete
-      slotFinalizeDeletion( NULL );
+      if( deletionPerformedByFilters )
+      {
+        applyFiltersDeleted();
+      }
+      else
+      {
+        slotFinalizeDeletion( NULL );
+      }
       return;
     }
     //if we have got a new password, it jumps to the end of the if-statement
@@ -365,7 +386,14 @@ void ConfigElem::slotMailDeleted( KIO::Job* job )
   {
     //unknown error, show message and cancel delete
     job->showErrorDialog();
-    slotFinalizeDeletion( NULL );
+    if( deletionPerformedByFilters )
+    {
+      applyFiltersDeleted();
+    }
+    else
+    {
+      slotFinalizeDeletion( NULL );
+    }
     return;
   }
   else
@@ -380,7 +408,14 @@ void ConfigElem::slotMailDeleted( KIO::Job* job )
     //if the list of mails to delete is empty, finalize the deletion and return
     if( MailsToDelete.empty() )
     {
-      commitDeletion();
+      if( deletionPerformedByFilters )
+      {
+        applyFiltersDeleted();
+      }
+      else
+      {
+        commitDeletion();
+      }
       return;
     }
   }
@@ -410,8 +445,18 @@ void ConfigElem::startKIOJob( const QString & path )
   //set options
   options.insert( "progress", "off" );
   options.insert( "pipelining", "off" );
-  options.insert( "auth", "TEXT" );
-  options.insert( "tls", "off" );
+
+  if( useTLS )
+    options.insert( "tls", "on" );
+  else
+    options.insert( "tls", "off" );
+
+  //Where is secure login?
+  //I have decided against a configurable secure login because the used POP3 kioslave
+  //always tries to login with APOP, if the server has sent a timestap (inside of the greeting string) for this authentification type.
+  //It just follows the auth-metadata, if the server doesn't support APOP (no timestamp inside of the greeting string).
+  //But I think, there is no server, which support a SASL authentification without also provide APOP.
+  //Ulrich Weigelt
 
   //set the given command and parameters
   m_url.setPath( path );
@@ -650,8 +695,12 @@ void ConfigElem::commitDownloading( )
   connect( pop3Job, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotFinalizeShowMail( KIO::Job* ) ) );
 }
 
-void ConfigElem::refreshMailList( )
+void ConfigElem::refreshMailList( FilterLog* log )
 {
+  //store pointer to log
+  if( log != NULL )
+    FLog = log;
+
   //return, if account is not active
   if( !isActive() )
   {
@@ -676,6 +725,14 @@ void ConfigElem::refreshMailList( )
   //set account state
   state = AccountRefreshing;
 
+  //init counter
+  if( !refreshPerformedByFilters )
+  {
+    nmbDeletedMailsLastRefresh = 0;
+    nmbMovedMailsLastRefresh = 0;
+    nmbIgnoredMails = 0;
+  }
+
   //the first step is to get the UIDs
   getUIDs();
 }
@@ -786,10 +843,10 @@ void ConfigElem::slotUIDsReceived( KIO::Job * job )
             //the mail is new
             isNew = true;
           }
-          else if( appConfig->keepNew() && m_pshowrecord->isNew( uid ) )
+          else if( ( appConfig->keepNew() || refreshPerformedByFilters ) && m_pshowrecord->isNew( uid ) )
           {
             //the mail is already in the old list
-            //but we will leave the state of formerly new mails, because the user wants it
+            //but we will leave the state of formerly new mails, because the user wants it or this refresh is performed by filters
             isNew = true;
           }
           else
@@ -812,8 +869,8 @@ void ConfigElem::slotUIDsReceived( KIO::Job * job )
   else
   {
     //we haven't received any UIDs. The account has no mails.
-    //Commit refresh.
-    commitRefresh();
+    //finalize the refresh
+    swapMailLists();
   }
 
 }
@@ -844,20 +901,16 @@ void ConfigElem::slotFinalizeRefresh( KIO::Job* )
   //stop timeout time
   pop3Timer->stop();
 
-  //delete old mail list
-  delete m_pshowrecord;
-
-  //assign the new list
-  if( tempMailList != NULL )
-    m_pshowrecord = tempMailList;
-  else
-    m_pshowrecord = new ShowRecord();
+  //unset the flag
+  refreshPerformedByFilters = false;
 
   //emit signal
   emit sigRefreshReady( m_strAccount );
 
   //set account state to idle
   state = AccountIdle;
+
+
 }
 
 void ConfigElem::commitRefresh( )
@@ -1055,8 +1108,8 @@ void ConfigElem::copyHeaders( )
     tempMailList->setHeader( *it, header );
   }
 
-  //commit refresh
-  commitRefresh();
+  //now we have the a complete new mail list
+  swapMailLists();
 }
 
 void ConfigElem::slotReceiveHeader( KIO::Job *, const QByteArray & data )
@@ -1199,3 +1252,508 @@ unsigned short int ConfigElem::getPort( ) const
   return m_url.port();
 }
 
+void ConfigElem::setTLS( bool tls )
+{
+  useTLS = tls;
+}
+
+bool ConfigElem::getTLS( ) const
+{
+  return useTLS;
+}
+
+void ConfigElem::reloadFilterSettings( )
+{
+  headerFilter.load();
+}
+
+void ConfigElem::applyFilters( )
+{
+  //are we executed by the MOVE routines?
+  if( !downloadActionsInvoked )
+  {
+    //this is the first call (at the current refresh cycle) of this methode
+    //we get the lists of mails to delete an move and call the MOVE routines if necessary
+
+    //OK, the filters were applied
+    filterApplied = true;
+
+    //order the mail list to apply the header filters
+    //it returns lists of mail numbers which shall be deleted or moved
+    //the marking will be done by the mail list itself
+    //the mail list removes all mails which shall be ignored itself
+    MailsToDelete.clear();
+    m_pshowrecord->applyHeaderFilter( &headerFilter, getAccountName(), MailsToDelete, MailsToDownload, nmbIgnoredMails, FLog );
+    nmbDeletedMailsLastRefresh += MailsToDelete.count();
+    nmbDeletedMailsLastStart += MailsToDelete.count();
+
+    //This part will be executed, if mails shall be downloaded
+    if( !MailsToDownload.empty() )
+    {
+      downloadActionsInvoked = true;
+      doDownloadActions();
+
+      //we quit this methode at this point, because after the bodies are downloaded and written this methode will recalled.
+      //At this time the else branch of this IF-statement will be executed and the methode continues
+      return;
+    }
+
+  }
+  else
+  {
+    //this is the second call (at the current refresh cycle) of this methode.
+    //it is called by the Move routines.
+    //the downloading of the mailbodies and writing it to the mailboxes has ended.
+    //A second call was just exceuted, if there was mails to move
+    downloadActionsInvoked = false;
+
+    //after an move error there are maybe some mails leftover in MailsToMove
+    MailsToDownload.clear();
+  }
+
+
+
+  //we have get the list of mails to delete and the all mails to move are written to its mailboxes
+  //now we delete this mails (the moved mails too)
+
+  if( !MailsToDelete.empty() )
+  {
+    //there are mails to delete
+    //we delete they
+    //after the delete cycle has done its job, it will call applyFiltersDeleted()
+    deletionPerformedByFilters = true;  //this is set to indicate the deletion is performed by filters and not by user
+                                        //the deletion methodes need it to decide on branch targets
+    deleteNextMail();
+  }
+  else
+  {
+    //if we need not to start a second refresh cycle (no mails was deleted or moved)
+    //we just commit the refresh and let the filter applied flag to false for the next regular refresh
+    commitRefresh();
+    filterApplied = false;
+  }
+}
+
+void ConfigElem::swapMailLists( )
+{
+  //delete old mail list
+  delete m_pshowrecord;
+
+  //assign the new list
+  if( tempMailList != NULL )
+    m_pshowrecord = tempMailList;
+  else
+    m_pshowrecord = new ShowRecord();
+
+  //if the filters were not applied yet, we do it now
+  //applyFilters() will either start a second refresh cycle if it did some deletions
+  //or call commitRefresh() to commit the refresh cycle.
+  //if the filters were already applied we commit the refresh.
+  if( filterApplied | !headerFilter.isActive() )
+  {
+    commitRefresh();
+    filterApplied = false;
+    return;
+  }
+  else
+  {
+    applyFilters();
+    return;
+  }
+}
+
+void ConfigElem::applyFiltersDeleted( )
+{
+  //unset the flag
+  deletionPerformedByFilters = false;
+
+  //start the second refresh cycle
+  refreshPerformedByFilters = true;
+  
+  //this sends a commit and restart the refresh
+  commitBeforeRefresh();
+  return;
+  //refreshMailList();
+}
+
+
+bool ConfigElem::writeToMailBox( const QString & mail, const QString & box )
+{
+  QDir mailDir( box );
+
+  //check whether the given path is a maildir
+  if( !isMailDir( mailDir ) )
+  {
+    //show an error message
+    KMessageBox::error( NULL, i18n( QString( "%1 is not a mailbox." ).arg( box ) ) );
+    return false;
+  }
+
+  //create unique file name according http://cr.yp.to/proto/maildir.html
+  QString partTime = QString::number( time( NULL ) );   //left part, output of time()
+
+  char hname[256];                                      //right part, the hostname
+  QString partHostname;
+  if( gethostname( hname, 255 ) == 0 )
+    partHostname = QString( hname );
+  else
+  {
+    //the hostname is not readable
+    //show an error message and exit
+    KMessageBox::error( NULL, i18n( QString( "Can't read the hostname of your computer. But KShowmail need it to write a mail into the mailbox." ) ) );
+    return false;
+  }
+
+  QString partPID = QString::number( getpid() );      //middle part, the PID
+
+  QString partCounter = QString::number( moveCounter++ );
+
+  QString uniqueName( partTime + "." + partPID + partCounter + "." + partHostname );
+
+  //build absolute path
+  mailDir.cd( "tmp" );
+  QString absFile = mailDir.filePath( uniqueName );
+
+  //and writing!
+  QFile file( absFile );
+  if( file.open( IO_WriteOnly ) )
+  {
+    QTextStream stream( &file );
+    stream << mail << endl;
+    file.close();
+  }
+  else
+  {
+    KMessageBox::detailedError( NULL, i18n( QString( "Could not file a mail to %1." ) ).arg( box ), i18n( file.errorString() ) );
+    return false;
+  }
+
+  //now we move it to the "new" subdirectory
+  mailDir.cdUp();
+  mailDir.cd( "new" );
+  QString absNewFile = mailDir.filePath( uniqueName );
+
+  if( rename( absFile.ascii(), absNewFile.ascii() ) == -1 )
+  {
+    KMessageBox::error( NULL, i18n( QString( "Could not move a mail from %1 to %2." ) ).arg( absFile ).arg( absNewFile ) );
+    return false;
+  }
+
+  //the writing was successful
+  return true;
+}
+
+void ConfigElem::doDownloadActions()
+{
+  //get first mail
+  getNextMailForDownloadActions();
+}
+
+bool ConfigElem::isMailDir( const QDir & path )
+{
+  //get a list of all subdirectories in this directory
+  const QStringList entries = path.entryList( QDir::Dirs | QDir::Readable | QDir::Writable | QDir::Hidden, QDir::Name | QDir::IgnoreCase | QDir::LocaleAware );
+
+  //a maildir folder must contains the folders "cur", "new" and "tmp"
+  bool curFound = false;
+  bool newFound = false;
+  bool tmpFound = false;
+
+  //iterate over all directories and look for the three necessary dirs
+  QStringList::const_iterator it = entries.begin();
+  while( it != entries.end() && !( curFound && newFound && tmpFound ) )
+  {
+    if( *it == "tmp" )
+      tmpFound = true;
+    else if( *it == "cur" )
+      curFound = true;
+    else if( *it == "new" )
+      newFound = true;
+
+    ++it;
+  }
+
+  return curFound && newFound && tmpFound;
+}
+
+void ConfigElem::getNextMailForDownloadActions()
+{
+    //if the list of mails to move is empty return to applyFilters
+  if( MailsToDownload.empty() )
+  {
+    applyFilters();
+    return;
+  }
+
+  //clear the class variable mailbody, which contains the downloaded mail body
+  mailbody.resize( 0 );
+
+  //start job
+  startKIOJob( QString( "/download/%1" ).arg( MailsToDownload.begin().key() ) );
+  connect( pop3Job, SIGNAL( data( KIO::Job*, const QByteArray & ) ), SLOT( slotDataMailBody( KIO::Job*, const QByteArray & ) ) );
+  connect( pop3Job, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotMailDownloadedForAction( KIO::Job* ) ) );
+
+}
+
+void ConfigElem::slotMailDownloadedForAction(KIO::Job * job)
+{
+  //stop timeout timer
+  pop3Timer->stop();
+
+  //check for errors
+  //if an error has occured, the download will be canceled
+  //or will ask for a new password
+  if( job->error() == KIO::ERR_COULD_NOT_LOGIN )
+  {
+    //login failed, ask for a new password
+    job->showErrorDialog();
+    bool res = assertPassword( true );
+    if( res == false )
+    {
+      //we have not got a new password; cancel delete
+      applyFilters();
+      return;
+    }
+    //if we have got a new password, jump to the end of the if-statement
+  }
+  else if( job->error() != 0 )
+  {
+    job->showErrorDialog();
+    applyFilters();
+    return;
+  }
+  else
+  {
+    //succesful download
+    //do action
+    MailToDownloadMap_Type::Iterator firstMail = MailsToDownload.begin();
+    int currentMailNumber = firstMail.key();  //get mail number
+    QString currentMailBox( firstMail.data().mailbox ); //get mailbox
+    QString mail( mailbody );                 //convert mailtext
+    FilterAction_Type action = firstMail.data().action; //get action
+
+    bool resultMove = false;    //TRUE - mail is written into the mailbox
+    bool resultSpam = false;  //TRUE - mail is Spam
+    bool deleteIt = false;    //TRUE - mail shall be deleted
+    bool resultAction = false;  //True - the action was succesful performed
+
+    switch( action )
+    {
+      case FActMove       : resultMove = writeToMailBox( mail, currentMailBox );
+                            //log entry is made by ShowRecordElem::applyHeaderFilter
+                            if( resultMove == true )
+                            {
+                              nmbMovedMailsLastRefresh++;
+                              nmbMovedMailsLastStart++;
+
+                              resultAction = true;
+                              deleteIt = true;
+                            }
+                            else
+                            {
+                              resultAction = false;
+                              deleteIt = false;
+                            }
+                            break;
+
+      case FActSpamcheck  : resultSpam = isSpam( mailbody );  //it is spam?
+                            if( resultSpam == true )          //yes, it is spam! Arrgghh! Torture it!!!
+                            {
+                              switch( appConfig->getSpamAction() )
+                              {
+                                case FActMove   : resultMove = writeToMailBox( mail, appConfig->getSpamMailbox() );
+                                                  if( resultMove == true )
+                                                  {
+                                                    nmbMovedMailsLastRefresh++;
+                                                    nmbMovedMailsLastStart++;
+
+                                                    if( FLog != NULL )
+                                                      m_pshowrecord->writeToMoveLog( FLog, currentMailNumber, getAccountName(), appConfig->getSpamMailbox() );
+                                                    resultAction = true;
+                                                    deleteIt = true;
+                                                  }
+                                                  else
+                                                  {
+                                                    resultAction = false;
+                                                    deleteIt = false;
+                                                  }
+                                                  break;
+
+                                case FActMark   : m_pshowrecord->setMarkAtNextViewRefresh( currentMailNumber );
+                                                  resultAction = true;
+                                                  deleteIt = false;
+                                                  break;
+
+                                case FActDelete : if( FLog != NULL )
+                                                    m_pshowrecord->writeToDeleteLog( FLog, currentMailNumber, getAccountName() );
+
+                                                  nmbDeletedMailsLastRefresh++;
+                                                  nmbDeletedMailsLastStart++;
+                                                  resultAction = true;
+                                                  deleteIt = true;
+                                                  break;
+
+                                default         : kdError() << "invalid action for spam mail" << endl;
+                                                  resultAction = false;
+                                                  deleteIt = false;
+                                                  break;
+
+                              }
+                            }
+                            else    //mail is not spam
+                            {
+                              resultAction = true;
+                              deleteIt = false;
+                            }
+                            break;
+
+      default             : deleteIt = false;
+                            resultAction = false;
+
+    }
+
+    if( resultAction == true )
+    {
+      //Action was successful
+      //remove this mail from the list
+      MailsToDownload.remove( firstMail );
+
+      //maybe add this mail to list of mails to delete
+      if( deleteIt )
+        MailsToDelete.append( currentMailNumber );
+    }
+    else
+    {
+      //Action was not successful
+      //returns to applyFilters() to continue the filtering
+      applyFilters();
+      return;
+    }
+
+
+    //if the list of mails is empty, return to applyFilters() to continue the filtering
+    if( MailsToDownload.empty() )
+    {
+      applyFilters();
+      return;
+    }
+  }
+
+
+  //show next mail in list
+  getNextMailForDownloadActions();
+}
+
+bool ConfigElem::isSpam( QByteArray mail ) const
+{
+  //check for a running spamassassin
+  if( !isSpamAssassinRunning() )
+  {
+    KMessageBox::information( NULL, i18n( "You want to check your mails for spam, but SpamAssassin is not running.\nKShowmail skips the spam check." ), i18n( "SpamAssassin is not running" ), "ConfigElemNoSpamAssassinRunning" );
+    return false;
+  }
+
+  //append an \0 at the end of the string
+  int size = mail.size();
+  if( mail[ size - 1 ] != '\0' )
+  {
+    mail.resize( size + 1 );
+    mail[ size ] = '\0';
+  }
+
+  //calls spmac and get an file pointer to stdin of it
+  FILE *write_fp;
+  write_fp = popen( "spamc -E", "w" );
+
+  //forward the mail to SpamAssassin
+  if( write_fp != NULL )
+  {
+    fwrite( mail.data(), sizeof( char), mail.size(), write_fp );
+
+    //check exit code of spamc and return result
+    int excode = pclose( write_fp );
+    if(  excode == 0 )
+      return false;
+    else
+      return true;
+  }
+  else
+  {
+    kdError() << "Could not call the command spamc of SpamAssassin." << endl;
+    return false;
+  }
+
+  return false;
+}
+
+bool ConfigElem::isSpamAssassinRunning( ) const
+{
+  FILE *read_fp;
+  char buffer[ BUFSIZ + 1 ];
+  int chars_read;
+  bool found = false;
+
+  memset( buffer, '\0', sizeof( buffer ) );
+  read_fp = popen( "ps -eo comm", "r" );
+  if( read_fp != NULL )
+  {
+    chars_read = fread( buffer, sizeof( char ), BUFSIZ, read_fp );
+    while( chars_read > 0 )
+    {
+      buffer[ chars_read - 1 ] = '\0';
+      QString output( buffer );
+      found = output.contains( NAME_SPAMASSASSIN_DAEMON ) > 0;
+      chars_read = fread( buffer, sizeof( char ), BUFSIZ, read_fp );
+    }
+    pclose( read_fp );
+  }
+
+  return found;
+}
+
+int ConfigElem::numberDeletedMailsLastRefresh( )
+{
+  return nmbDeletedMailsLastRefresh;
+}
+
+int ConfigElem::numberDeletedMailsStart( )
+{
+  return nmbDeletedMailsLastStart;
+}
+
+int ConfigElem::numberMovedMailsLastRefresh( )
+{
+  return nmbMovedMailsLastRefresh;
+}
+
+int ConfigElem::numberMovedMailsStart( )
+{
+  return nmbMovedMailsLastStart;
+}
+
+int ConfigElem::numberIgnoredMails( )
+{
+  return nmbIgnoredMails;
+}
+
+QStringList ConfigElem::getSelectedSenders( ) const
+{
+  return m_pshowrecord->getSelectedSenders();
+}
+
+
+
+
+void ConfigElem::commitBeforeRefresh()
+{
+  //start job to commit
+  startKIOJob( QString( "/commit" ) );
+  connect( pop3Job, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotCommitBeforeRefreshDone( KIO::Job* ) ) );
+
+}
+
+void ConfigElem::slotCommitBeforeRefreshDone(KIO::Job *)
+{
+  //after a commit was send, we start a new refresh cyle
+  refreshMailList();
+}
\ No newline at end of file
