@@ -18,6 +18,12 @@
 #ifndef CONFIGELEM_H
 #define CONFIGELEM_H
 
+//C headers
+#include <time.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <stdio.h>
+
 //QT headers
 #include <qstring.h>
 #include <qlistview.h>
@@ -25,6 +31,9 @@
 #include <qobject.h>
 #include <qtimer.h>
 #include <qregexp.h>
+#include <qdir.h>
+#include <qfile.h>
+#include <qtextstream.h>
 
 //KDE headers
 #include <kurl.h>
@@ -46,6 +55,8 @@
 #include "kshowmailview.h"
 #include "encryption.h"
 #include "kwalletaccess.h"
+#include "headerfilter.h"
+#include "filterlog.h"
 
 
 
@@ -228,6 +239,19 @@ class ConfigElem : public QObject {
     */
    QString getProtocol( bool upperCase = false ) const;
 
+   /**
+    * Enable or disable TLS.
+    * @param tls TRUE: use TLS; FALSE: don't use TLS
+    */
+   void setTLS( bool tls );
+
+   /**
+    * Returns whether TLS is enabled.
+    * @return TRUE: TLS is enabled
+    * @return FALSE: TLS is disabled
+    */
+   bool getTLS() const;
+
    /**
     * Sets the port.
     * @param port the port
@@ -282,12 +306,6 @@ class ConfigElem : public QObject {
    */
   void readStoredMails( QDomElement& parent );
 
-  /**
-   * Apply the filters to the stored mails.
-   * @see ShowRecordElem::applyFilters
-   */
-  void applyFilters();
-
   /**
    * Returns the number of stored mails.
    * @return number of mails
@@ -340,13 +358,14 @@ class ConfigElem : public QObject {
   bool hasSelectedMails();
 
   /**
-   * Refrehes the mail list.
+   * Refreshes the mail list.
    * This just starts the refresh and returns after then.
    * When the refresh is ready, the signal sigRefreshReady
    * will be emitted.
+   * @param log Pointer to the filter log
    * @see sigRefreshReady
    */
-  void refreshMailList();
+  void refreshMailList( FilterLog* log = NULL );
 
   /**
    * Returns the number of new mails.
@@ -398,6 +417,48 @@ class ConfigElem : public QObject {
    */
   void printSetup() const;
 
+  /**
+   * Reloads the settings of the filters.
+   * It just calls the load() methode of the header filter.
+   */
+  void reloadFilterSettings();
+
+    /**
+     * Returns the number of deleted mails by last refresh.
+     * @return number of deleted mails by last refresh
+     */
+  int numberDeletedMailsLastRefresh();
+
+    /**
+     * Returns the number of deleted mails since start.
+     * @return number of deleted mails since start
+     */
+  int numberDeletedMailsStart();
+
+    /**
+     * Returns the number of moved mails by last refresh.
+     * @return number of moved mails by last refresh
+     */
+  int numberMovedMailsLastRefresh();
+
+    /**
+     * Returns the number of moved mails since start.
+     * @return number of moved mails since start
+     */
+  int numberMovedMailsStart();
+
+    /**
+     * Returns the number of ignored mails by last refresh.
+     * @return number of ignored mails by last refresh
+     */
+  int numberIgnoredMails();
+
+  /**
+   * Returns the senders of the selected mails
+   * @return senders of the selected mails
+   */
+  QStringList getSelectedSenders() const;
+
 
   private:
 
@@ -407,6 +468,12 @@ class ConfigElem : public QObject {
      */
     ConfigList* appConfig;
 
+    /**
+     * Pointer to the filter log.
+     * Will be set by refreshMailList() at every refresh.
+     */
+    FilterLog* FLog;
+
     /**
      * State of the account.
      */
@@ -441,12 +508,22 @@ class ConfigElem : public QObject {
      */
     ShowRecord* m_pshowrecord;
 
-
     /**
      * Uniform Resource Locator of the account on the pop3 server.
      */
     KURL m_url;
 
+    /**
+     * Type of secure login.
+     * See constants.h for valid values (CONFIG_VALUE_ACCOUNT_SECLOGIN_...)
+     */
+    int secureLoginType;
+
+    /**
+     * False: TLS is enabled
+     */
+    bool useTLS;
+
     /**
      * Contains a downloaded mail body.
      * Will be set by slotDataMailBody() and read by slotBodyDownloaded().
@@ -455,13 +532,22 @@ class ConfigElem : public QObject {
 
    /**
     * Contains the numbers of mails which will be deleted.
-    * Set by deleteSelectedMails() and used by deleteNextMail() to get
+    * Set by deleteSelectedMails() and applyFilters() and used by deleteNextMail() to get
     * the number of the next mail to delete.
-    * @see deleteSelectedMails
-    * @see deleteNextMail
+    * @see deleteSelectedMails()
+    * @see deleteNextMail()
+    * @see applyFilters()
     */
     MailNumberList_Type MailsToDelete;
 
+    /**
+     * Contains the numbers of the mails shall be moved for actions like moving to mailbox or spam check.
+     * Contains also some parameters (like mailbox) about the action which have to be performed.
+     * Set and used by applyFilters().
+     * @see applyFilters()
+     */
+    MailToDownloadMap_Type MailsToDownload;
+
     /**
      * Contains the numbers of mails for which the bodies will be shown.
      * Set by showSelectedMails() and used by showNextMail() to get
@@ -516,6 +602,68 @@ class ConfigElem : public QObject {
      */
     QByteArray receivedHeader;
 
+    /**
+     * Every account has is own header filter instance.
+     */
+    HeaderFilter headerFilter;
+
+    /**
+     * This is a flag whether the filters was applied during a refresh cycle.
+     * If the some mails was deleted (recommend by the filter), we need a actual
+     * list of mails from the server. Therefore a second refresh cycle will be started.
+     * To avoid a never-ending loop this flag will be set to TRUE by applyFilters().
+     * If the second refresh is running this one will not invoke the filters.
+     */
+    bool filterApplied;
+
+    /**
+     * This flag is set by applyFilters() to indicate the deletion cycle is performed by filters and not by user.
+     * The deletion methodes need it to decide on branch targets
+     */
+    bool deletionPerformedByFilters;
+
+    /**
+     * This flag is set by applyFiltersDeleted() to indicate the refresh cycle is the second one after a deletion which was performed by filters.
+     * The refresh cycle need it to hold the remaining mails as new.
+     */
+    bool refreshPerformedByFilters;
+
+    /**
+     * This flag is set to TRUE by applyFilters() to indicate the mailbodies are downloading and writing to the mailboxes or spam checking.
+     * When this is ended, applyFilters() will be called again and it reset this flag
+     */
+    bool downloadActionsInvoked;
+
+    /**
+     * Counter for moving to create an unique file name
+     */
+    int moveCounter;
+
+    /**
+     * Number of deleted mails by last refresh.
+     */
+    int nmbDeletedMailsLastRefresh;
+
+    /**
+     * Number of deleted mails since start.
+     */
+    int nmbDeletedMailsLastStart;
+
+    /**
+     * Number of moved mails by last refresh.
+     */
+    int nmbMovedMailsLastRefresh;
+
+    /**
+     * Number of moved mails since start.
+     */
+    int nmbMovedMailsLastStart;
+
+    /**
+     * Number of ignored mails by last refresh.
+     */
+    int nmbIgnoredMails;
+
     /**
      * Initializes the account.
      */
@@ -612,12 +760,36 @@ class ConfigElem : public QObject {
     void getNextHeader();
 
     /**
-     * Fourth and last step of the refresh cycle.
+     * Fourth step of the refresh cycle.
      * Copies the known headers of the old mails from the
      * old mail list to the new one.
+     * At the end it invokes swapMailLists().
      */
     void copyHeaders();
 
+    /**
+     * Sixth step of the refresh cycle.
+     * Applies the filters to the mails in the mail list.
+     * Invoked by swapMailLists().
+     * This methode maybe starts a new refresh cycle because after a deletion or moving we need a
+     * actual list of mails on the server.
+     * To avoid a never-ending loop you must not call this methode during the second refresh cycle.
+     * Therefore it sets filterApplied to TRUE.
+     * @see filterApplied
+     * @see applyFiltersDeleted()
+     */
+    void applyFilters();
+
+    /**
+     * If there are mails to delete by filters applyFilters will call the regular deletion cycle of this class with set byFilter-flag.
+     * Therefore the deletion will not branch to commitDeletion() but to this methode.
+     * This performs a second refresh cycle to get an effective mail list from the server.
+     * This new cycle will not apply the filters again, because the flag filterApplied is set to TRUE.
+     * @see applyFilters()
+     * @see filterApplied
+     */
+    void applyFiltersDeleted();
+
     /**
      * Cancels the refresh. Will be invoked, when a pop3 job has
      * reported an error.
@@ -628,10 +800,71 @@ class ConfigElem : public QObject {
      * Commits the refresh.
      * Starts a pop3 job that will do the commit. Its signal
      * result is connected with slotFinalizeRefresh().
+     * Invoked by copyHeaders() if the filters was applied or applyFilters()
+     * if not second refresh was necessary.
      * @see slotFinalizeRefresh()
+     * @see applyFilters()
      */
     void commitRefresh();
 
+    /**
+     * Send a commit and restart the refresh cycle
+     * We restart the refresh to get a fresh maillist after a deletion performed by the filter.
+     * We need a commit before because the mails server only after a commit reorders the mail numbers
+     */
+    void commitBeforeRefresh();
+
+    /**
+     * Fifth step of the refresh cycle.
+     * Copies the pointer to the temporary list to m_pshowrecord.
+     * Invoked by copyHeaders() and slotUIDsReceived().
+     * The temporary list will be the current mail list.
+     * At the end it invokes commitRefresh() if the filters were applied
+     * (filterApplied == TRUE) or deactive or it invokes
+     * applyFilters() if the filters were not
+     * applied (filterApplied == FALSE).
+     * @see m_pshowrecord
+     * @see copyHeaders()
+     * @see slotUIDsReceived()
+     * @see commitRefresh()
+     * @see filterApplied
+     */
+    void swapMailLists();
+
+    /**
+     * Does all filter actions for which we have to download the mails
+     * These are:
+     * Moving, spam check
+     * It doesn't removes they from the server.
+     */
+    void doDownloadActions();
+
+    /**
+     * Gets the body of the first mail in MailsToDownload.
+     * After a succesful download and writing into the mailbox this
+     * mail will be removed from the list by slotMailDownloadedforDownloadActions() and this
+     * method will be invoked again.
+     * If the list is empty, it will call applyFilters() to continue the filtering.
+     * @see applyFilters()
+     * @see slotMailDownloadedforDownloadActions()
+     */
+    void getNextMailForDownloadActions();
+
+    /**
+     * Returns whether the given directory is a maildir directory
+     * @param path the directory for test
+     * @return TRUE - directory is a maildir directory
+     */
+    bool isMailDir( const QDir& path );
+
+    /**
+     * Writes the given mail into the mailbox.
+     * @param mail the Mail
+     * @param box path to the mailbox
+     * @return TRUE - Writing was succesful
+     */
+    bool writeToMailBox( const QString& mail, const QString& box );
+
   private slots:
 
     /**
@@ -679,13 +912,29 @@ class ConfigElem : public QObject {
      */
     void slotBodyDownloaded( KIO::Job* job );
 
+    /**
+     * Connected with signal result of the launched pop3 job to
+     * download a mail to write it into a mailbox or do a spam check.
+     * Maybe it puts the number of the downloaded mail into the list of mails to delete (MailsToDelete).
+     * Removes the first mail from MailsToMove and invokes getNextMailForMove()
+     * again to get and write the next mail.
+     * If the list is empty after it has removed the first item, it will call
+     * applyFilters() to continue the filtering.
+     * If an error is occured the current mail will not put into MailsToDelete. And it returns
+     * to applyFilters() immediately.
+     * @param job job which has emit the result signal
+     * @see MailsToDownload
+     * @see getNextMailForDownloadActions()
+     * @see applyFilters()
+     */
+    void slotMailDownloadedForAction( KIO::Job* job );
+
     /**
      * Connected with signal data of the launched pop3 job to
      * download a mail body.
      * Receives the downloaded data from the job. This lot will be called
      * several times during the donwload. At the last one the array is empty.
      * It extracts the body from the downloaded data and stores it in DownloadedMailBody.
-     * @param job job which has emit the data signal
      * @param datas the downloaded mail
      */
     void slotDataMailBody( KIO::Job*, const QByteArray & datas );
@@ -781,6 +1030,14 @@ class ConfigElem : public QObject {
      * @param data received data, which contains a header part
      */
     void slotReceiveHeader( KIO::Job*, const QByteArray& data );
+    
+    /**
+     * Connected with the result signal of the pop3 job launched by commitBeforeRefresh()
+     * Restarts a second refresh cycle after a deletion performed by the filter.
+     * @see commitBeforeRefresh
+     * @see applyFiltersDeleted
+     */
+    void slotCommitBeforeRefreshDone( KIO::Job* );
 
 
   signals:
@@ -819,5 +1076,22 @@ class ConfigElem : public QObject {
      */
     void sigRefreshReady( QString account );
 
+  protected:
+
+    /**
+     * Forwards the given mail to SpamAssassin and returns the test result
+     * @param mail the mail to test
+     * @return TRUE - mail is spam
+     */
+    bool isSpam( QByteArray mail ) const;
+
+    /**
+     * Looks for a running spamd daemon of SpamAssassin.
+     * @return TRUE - SpamAssassin is running
+     */
+    bool isSpamAssassinRunning() const;
+
+
+
 };
 #endif
