@@ -29,7 +29,6 @@
 #include <kdebug.h>
 
 #include "configlist.h"
-#include "filter.h"
 
 ConfigList::ConfigList() : QObject()
 {
@@ -57,6 +56,7 @@ ConfigList::ConfigList() : QObject()
     m_nInitTimer = DEFAULT_INITIAL_TIME;
     m_nIntervalTimer = DEFAULT_INTERVAL_TIME;
     m_nPop3Timer = DEFAULT_TIMEOUT_TIME;
+
 }
 
 int ConfigList::compareItems( QCollection::Item item1, QCollection::Item item2 )
@@ -107,7 +107,7 @@ void ConfigList::saveOptions ()
 
   //save XML document
   QCString str = doc.toCString(); //convert XML document to a string
-  QString cachefilename = locateLocal( "config", "kshowmail.xml" ); //get file path
+  QString cachefilename = locateLocal( "config", MAIL_FILE ); //get file path
   KSaveFile file( cachefilename, 0600 );  //create file
 
   if( file.status() != 0 )
@@ -182,21 +182,6 @@ void ConfigList::playSound (const char* file)
 	KAudioPlayer::play(file);
 }
 
-
-void ConfigList::applyFilters ()
-{
-  if (Filter::_status != Filter::off)
-  {
-    for (ConfigElem* pElem = first(); pElem; pElem = next())
-    {
-      if (pElem->isActive() )
-      {
-        pElem->applyFilters ();
-      }
-    }
-  }
-}
-
 int ConfigList::getRefreshTimeInterval( ) const
 {
   return m_nIntervalTimer;
@@ -519,7 +504,7 @@ void ConfigList::slotMessageWindowClosed( )
     emit sigAllMessageWindowsClosed();
 }
 
-void ConfigList::refreshMailLists( )
+void ConfigList::refreshMailLists( FilterLog* log )
 {
   QPtrListIterator<ConfigElem> it( *this );   //to iterate over all accounts
   ConfigElem* account;                        //account to process
@@ -555,7 +540,7 @@ void ConfigList::refreshMailLists( )
   it.toFirst();
   while( ( account = it.current() ) != NULL )
   {
-    account->refreshMailList();
+    account->refreshMailList( log );
 
     //get next account
     ++it;
@@ -787,6 +772,31 @@ void ConfigList::refreshSetup( KListView* view )
   config->setGroup( CONFIG_GROUP_VIEW );
   m_bAllowHTML = config->readBoolEntry( CONFIG_ENTRY_VIEW_USE_HTML, DEFAULT_VIEW_USE_HTML );
 
+  //read Spam configs
+  config->setGroup( CONFIG_GROUP_SPAMCHECK );
+  int intSpamAction = config->readNumEntry( CONFIG_ENTRY_SPAMCHECK_ACTION, DEFAULT_SPAMCHECK_ACTION );
+
+  switch( intSpamAction )
+  {
+    case CONFIG_VALUE_SPAMCHECK_ACTION_DELETE       : spamAction = FActDelete; break;
+    case CONFIG_VALUE_SPAMCHECK_ACTION_MARK         : spamAction = FActMark; break;
+    case CONFIG_VALUE_SPAMCHECK_ACTION_MOVE         : spamAction = FActMove; break;
+    default                                         :
+      kdError() << "Invalid value in " << CONFIG_ENTRY_SPAMCHECK_ACTION << ". Set default value." << endl;
+      switch( DEFAULT_SPAMCHECK_ACTION )
+      {
+        case CONFIG_VALUE_SPAMCHECK_ACTION_DELETE       : spamAction = FActDelete; break;
+        case CONFIG_VALUE_SPAMCHECK_ACTION_MARK         : spamAction = FActMark; break;
+        case CONFIG_VALUE_SPAMCHECK_ACTION_MOVE         : spamAction = FActMove; break;
+        default                                         : spamAction = FActMark; break;
+      }
+
+  }
+
+  if( spamAction == FActMove )
+    spamMailbox = config->readEntry( CONFIG_ENTRY_SPAMCHECK_MOVE_MAILBOX, DEFAULT_SPAMCHECK_ACTION_MOVE_MAILBOX );
+
+
   //read account configuration and setup accounts
   //---------------------------------------------
 
@@ -838,7 +848,27 @@ void ConfigList::refreshSetup( KListView* view )
     config->setGroup( *it );
 
     acc->setHost( config->readEntry( CONFIG_ENTRY_ACCOUNT_SERVER, DEFAULT_ACCOUNT_SERVER ) );
-    acc->setProtocol( config->readEntry( CONFIG_ENTRY_ACCOUNT_PROTOCOL, DEFAULT_ACCOUNT_PROTOCOL ).lower() );
+
+    //set protocol and security
+    //if the read protocol is POP3 and SSL is enabled, the account protocol will set to "pop3s"
+    QString proto = config->readEntry( CONFIG_ENTRY_ACCOUNT_PROTOCOL, DEFAULT_ACCOUNT_PROTOCOL ).lower();
+    int secureTransfer = config->readNumEntry( CONFIG_ENTRY_ACCOUNT_SECTRANSFER, DEFAULT_ACCOUNT_SECTRANSFER );
+
+    if( proto == "pop3" &&  secureTransfer == CONFIG_VALUE_ACCOUNT_SECTRANSFER_SSL )
+    {
+      acc->setProtocol( "pop3s" );
+    }
+    else
+    {
+      acc->setProtocol(  proto );
+    }
+
+    if( secureTransfer == CONFIG_VALUE_ACCOUNT_SECTRANSFER_TLS )
+      acc->setTLS( true );
+    else
+      acc->setTLS( false );
+
+
     acc->setPort( config->readNumEntry( CONFIG_ENTRY_ACCOUNT_PORT, DEFAULT_ACCOUNT_PORT_POP3 ) );
     acc->setUser( config->readEntry( CONFIG_ENTRY_ACCOUNT_USER, DEFAULT_ACCOUNT_USER ) );
     acc->setActive( config->readBoolEntry( CONFIG_ENTRY_ACCOUNT_ACTIVE, DEFAULT_ACCOUNT_ACTIVE ) );
@@ -866,6 +896,8 @@ void ConfigList::refreshSetup( KListView* view )
         acc->setPassword( QString::null );
     }
 
+    //order the account to reloads its filter setup
+    acc->reloadFilterSettings();
   }
 
   //connect the signals of the accounts with ConfigList
@@ -1033,4 +1065,139 @@ void ConfigList::readStoredMails( )
   file.close();
 }
 
+FilterAction_Type ConfigList::getSpamAction( )
+{
+  return spamAction;
+}
+
+QString ConfigList::getSpamMailbox( )
+{
+  return spamMailbox;
+}
+
+int ConfigList::numberDeletedMailsLastRefresh( )
+{
+  ConfigElem* account = NULL;
+  QPtrListIterator<ConfigElem> it( *this ); //iterator for the account list
+  int number = 0;
+
+  //iterate over all accounts
+  while( ( account = it.current() ) != NULL )
+  {
+    //increment iterator to next account
+    ++it;
+
+    number += account->numberDeletedMailsLastRefresh();
+  }
+
+  return number;
+}
+
+int ConfigList::numberDeletedMailsStart( )
+{
+  ConfigElem* account = NULL;
+  QPtrListIterator<ConfigElem> it( *this ); //iterator for the account list
+  int number = 0;
+
+  //iterate over all accounts
+  while( ( account = it.current() ) != NULL )
+  {
+    //increment iterator to next account
+    ++it;
+
+    number += account->numberDeletedMailsStart();
+  }
+
+  return number;
+}
+
+int ConfigList::numberMovedMailsLastRefresh( )
+{
+  ConfigElem* account = NULL;
+  QPtrListIterator<ConfigElem> it( *this ); //iterator for the account list
+  int number = 0;
+
+  //iterate over all accounts
+  while( ( account = it.current() ) != NULL )
+  {
+    //increment iterator to next account
+    ++it;
+
+    number += account->numberMovedMailsLastRefresh();
+  }
+
+  return number;
+}
+
+int ConfigList::numberMovedMailsStart( )
+{
+  ConfigElem* account = NULL;
+  QPtrListIterator<ConfigElem> it( *this ); //iterator for the account list
+  int number = 0;
+
+  //iterate over all accounts
+  while( ( account = it.current() ) != NULL )
+  {
+    //increment iterator to next account
+    ++it;
+
+    number += account->numberMovedMailsStart();
+  }
+
+  return number;
+}
+
+int ConfigList::numberIgnoredMails( )
+{
+  ConfigElem* account = NULL;
+  QPtrListIterator<ConfigElem> it( *this ); //iterator for the account list
+  int number = 0;
+
+  //iterate over all accounts
+  while( ( account = it.current() ) != NULL )
+  {
+    //increment iterator to next account
+    ++it;
+
+    number += account->numberIgnoredMails();
+  }
+
+  return number;
+}
+
+QStringList ConfigList::getSelectedSenders( ) const
+{
+  QStringList senders;                       //contains all senders
+  QPtrListIterator<ConfigElem> it( *this );   //to iterate over all accounts
+  ConfigElem* account;                        //current account
+
+  while( ( account = it.current() ) != NULL )
+  {
+    //get senders of the current account and append them to the list
+    senders += account->getSelectedSenders();
+
+    //get next account
+    ++it;
+  }
+
+  return senders;
+}
+
+void ConfigList::refreshFilterSetup( )
+{
+  ConfigElem* account;                      //account
+  QPtrListIterator<ConfigElem> it( *this ); //iterator for the account list
+
+  //iterate over all accounts
+  while( ( account = it.current() ) != NULL )
+  {
+    //increment iterator to next account
+    ++it;
+
+    //reload filter setup of the current account
+    account->reloadFilterSettings();
+  }
+
+}
+
 
