@@ -39,7 +39,9 @@
 #include "types.h"
 #include "encryption.h"
 #include "kwalletaccess.h"
-
+#include "filteritem.h"
+#include "headerfilter.h"
+#include "filterlog.h"
 
 using namespace Constants;
 using namespace Types;
@@ -78,6 +80,11 @@ class ConfigList : public QObject, public QPtrList<ConfigElem>
      */
     void refreshSetup( KListView* view );
 
+    /**
+     * Reloads the filter settings
+     */
+    void refreshFilterSetup();
+
     /**
     * Returns the auto refresh time interval.
     * @return refresh time interval [minutes]
@@ -98,7 +105,7 @@ class ConfigList : public QObject, public QPtrList<ConfigElem>
 
     /**
      * Sets whether the user has to confirm a deletion.
-     * @param confirm TRUE - user has to confirm; FALSE - the user does not have to confirm 
+     * @param confirm TRUE - user has to confirm; FALSE - the user does not have to confirm
      */
     void setConfirmDeletion( bool confirm );
 
@@ -159,7 +166,6 @@ class ConfigList : public QObject, public QPtrList<ConfigElem>
      */
     bool allowHTML() const;
 
-
     /**
      * Returns whether there are active accounts.
      * Doesn't set the current account.
@@ -234,8 +240,9 @@ class ConfigList : public QObject, public QPtrList<ConfigElem>
      * This just starts the refresh and returns after then. When all
      * accounts have refreshed their mail list the signal sigRefreshReady will be
      * emitted.
+     * @param log Pointer to the filter log.
      */
-    void refreshMailLists();
+    void refreshMailLists( FilterLog* log = NULL );
 
     /**
      * Returns the number of new mails.
@@ -360,13 +367,59 @@ class ConfigList : public QObject, public QPtrList<ConfigElem>
      */
     void readStoredMails();
 
+    /**
+     * Returns the action to used on spam mails
+     * @return action to used on spam mails
+     */
+     FilterAction_Type getSpamAction();
+
+    /**
+     * Returns the mailbox for spam mails
+     * @return mailbox for spam
+     */
+     QString getSpamMailbox();
+
+    /**
+      * Returns the number of deleted mails by last refresh.
+      * @return number of deleted mails by last refresh
+      */
+     int numberDeletedMailsLastRefresh();
+
+    /**
+      * Returns the number of deleted mails since start.
+      * @return number of deleted mails since start
+      */
+     int numberDeletedMailsStart();
+
+    /**
+      * Returns the number of moved mails by last refresh.
+      * @return number of moved mails by last refresh
+      */
+     int numberMovedMailsLastRefresh();
+
+    /**
+      * Returns the number of moved mails since start.
+      * @return number of moved mails since start
+      */
+     int numberMovedMailsStart();
+
+    /**
+      * Returns the number of ignored mails by last refresh.
+      * @return number of ignored mails by last refresh
+      */
+     int numberIgnoredMails();
+
+    /**
+     * Returns the senders of the selected mails
+     * @return senders of the selected mails
+     */
+     QStringList getSelectedSenders() const;
 
 	bool setItem (const char* item);
 	void setList (QListView* list);
 	void beep ();
 	void playSound ();
 	void playSound (const char* file);
-  void applyFilters();
 
 
   private:
@@ -529,6 +582,16 @@ class ConfigList : public QObject, public QPtrList<ConfigElem>
      */
     bool m_bKeepNew;
 
+    /**
+     * Mailbox for Spam, if configured
+     */
+     QString spamMailbox;
+
+    /**
+     * Action which shall be done, if the mail is Spam
+     */
+      FilterAction_Type spamAction;
+
 
   protected:
 
@@ -665,6 +728,7 @@ class ConfigList : public QObject, public QPtrList<ConfigElem>
      * Will be emitted, when all accounts have refreshed their mail list.
      */
     void sigRefreshReady();
+
 };
 
 #endif
