@@ -29,7 +29,7 @@ KShowMailApp::KShowMailApp()
   initActions();
 
   // create GUI from ui resource file
-  setXMLFile( "kshowmailui.rc" );
+  setXMLFile( "kshowmailui.rc", true );
   createGUI();
 
   //the main window settings will be saved automatically
@@ -55,8 +55,6 @@ KShowMailApp::KShowMailApp()
   //read the setup
   m_ConfigList.refreshSetup( m_pView->m_pListAccounts );
 
-  readOptions();
-
   //load stored mails
   m_ConfigList.readStoredMails();
 
@@ -73,11 +71,13 @@ KShowMailApp::KShowMailApp()
   connect( &m_ConfigList, SIGNAL( sigAllMessageWindowsClosed() ), this, SLOT( slotWaitingCursor() ) );
   connect( &m_ConfigList, SIGNAL( sigRefreshReady() ), this, SLOT( slotRefreshReady() ) );
 
+  //show welcome message
+  KMessageBox::information( NULL, i18n( "Thank You for using KShowmail.\nPlease use the feedback dialog to tell us your experience with this program." ), i18n("Welcome"), "WelcomeMessage" );
+
   // this starts the one-second-interval timer
   // this is a QObject method
   startTimer (1000);
 
-
   //we are ready to take off! Lets initiate the first refresh (if desired)
   initFirstRefresh();
 }
@@ -100,10 +100,10 @@ void KShowMailApp::timerEvent (QTimerEvent *)
     QTime time;
     time = time.addSecs (m_nSecondsToGo--);
     QCString msg (i18n("Autorefresh: %1").arg (time.toString()));
-    statusBar()->changeItem(msg, 3);
+    statusBar()->changeItem( msg, STATUSBAR_FIELD_NEXT_REFRESH );
   }
   else
-    statusBar()->changeItem("", 3);
+    statusBar()->changeItem( "", STATUSBAR_FIELD_NEXT_REFRESH );
 
   // if busy, animate the traybar
   if ( m_state == refreshing || m_state == deleting )
@@ -126,7 +126,6 @@ void KShowMailApp::timerEvent (QTimerEvent *)
 void KShowMailApp::initActions ()
 {
   // file menu
-  m_actionSave = KStdAction::saveOptions( this, SLOT( slotSaveOptions() ), actionCollection() );
   KStdAction::quit( this, SLOT( slotFileQuit() ), actionCollection() );
 
   // action menu
@@ -135,10 +134,12 @@ void KShowMailApp::initActions ()
   new KAction( i18n( "Show &complete highlighted messages" ), "letter-open", Key_F3, this, SLOT( slotShowMessage() ), actionCollection(), "show_complete" );
   new KAction( i18n( "&Delete highlighted messages" ), "eraser", Key_Delete, this, SLOT( slotDelete() ), actionCollection(), "delete" );
   new KAction( i18n( "S&top current transfer" ), "stop", 0, this, SLOT( slotStop() ), actionCollection(), "stop" );
+  new KAction( i18n( "Show Filter Log" ), "log", Key_F4, this, SLOT( slotShowFilterLog() ), actionCollection(), "show_filterlog" );
+  new KAction( i18n( "Add sender to whitelist" ), "add_user", 0, this, SLOT( slotAddToWhitelist() ), actionCollection(), "addWhitelist" );
+  new KAction( i18n( "Add sender to blacklist" ), "delete_user", 0, this, SLOT( slotAddToBlacklist() ), actionCollection(), "addBlacklist" );
 
   // setup menu
   KStdAction::preferences( this, SLOT( slotSetup() ), actionCollection() );
-  new KAction( i18n( "Setup &filters" ), "filter", 0, this, SLOT( slotSetupFilters() ), actionCollection(), "filters" );
   KStdAction::configureToolbars( this, SLOT( slotEditToolbars() ), actionCollection() );
 
   //create menu items for toggle tool and status bar
@@ -152,18 +153,18 @@ void KShowMailApp::initActions ()
   new KAction(i18n( "Setup &account" ), "tool", 0, this, SLOT( slotSetupAccount() ), actionCollection(), "setupAccount" );
 }
 
+
 void KShowMailApp::initStatusBar()
 {
-  ///////////////////////////////////////////////////////////////////
-  // STATUSBAR
-  QCString user = getenv( "LOGNAME" );
-  QCString server = getenv( "HOSTNAME" );
-  QCString  mesg (QString ("%1@%2").arg(user).arg(server));
+  statusBar()->insertItem( i18n( "Ready." ), STATUSBAR_FIELD_STATE );
+  statusBar()->insertItem( "", STATUSBAR_FIELD_FILTER, 1 );
+  statusBar()->setItemAlignment( STATUSBAR_FIELD_FILTER, AlignLeft | AlignVCenter );
+  statusBar()->insertItem( "", STATUSBAR_FIELD_NEXT_REFRESH );
+  statusBar()->insertItem( "", STATUSBAR_FIELD_LAST_REFRESH );
 
-  statusBar()->insertItem(i18n("Ready."), 1);
-  statusBar()->insertItem(mesg, 2, 1);
-  statusBar()->insertItem("", 3);
-  statusBar()->insertItem("", 4);
+   QToolTip::add( statusBar(), i18n( "Shows the number of deleted, moved or ignored mails by the filter.\nThe positions denotes:\nby last refresh / since application start / listed by the log" ) );
+
+  refreshFilterStatusBar();
 }
 
 void KShowMailApp::initDocument()
@@ -189,22 +190,12 @@ void KShowMailApp::slotSaveOptions()
 
   //order application parts to save their setup
   m_pView->saveOptions( config );
-  Filter::saveOptions( config );
   m_ConfigList.saveOptions ();
+  fLog.save();
 
   //save config to file
   config->sync ();
 
-  //disable save action
-  m_actionSave->setEnabled( false );
-}
-
-void KShowMailApp::readOptions()
-{
-  KConfig* config = KGlobal::config();
-
-  Filter::readOptions (config);
-  m_actionSave->setEnabled (false);
 }
 
 bool KShowMailApp::queryClose()
@@ -338,7 +329,7 @@ void KShowMailApp::slotRefresh()
   QApplication::setOverrideCursor( Qt::waitCursor );
 
   //order the account list to refresh its mail lists
-  m_ConfigList.refreshMailLists();
+  m_ConfigList.refreshMailLists( &fLog );
 
   return;
 }
@@ -399,6 +390,9 @@ void KShowMailApp::slotRefreshView ()
   //show message in status bar
   long totalSize = m_ConfigList.getTotalSize();
   slotStatusMsg( i18n( "%1 message(s) with a total of %2 bytes are waiting" ).arg( numberMails ).arg( totalSize ) );
+
+  //refresh filter view in the status bar
+  refreshFilterStatusBar();
 }
 
 void KShowMailApp::slotDelete()
@@ -491,10 +485,10 @@ void KShowMailApp::slotStatusMsg(const QString& text)
   statusBar()->clear();
 
   //set given text
-  statusBar()->changeItem(text, 1);
+  statusBar()->changeItem( text, STATUSBAR_FIELD_STATE );
 
   //set current time
-  statusBar()->changeItem(sTime, 4);
+  statusBar()->changeItem( i18n( "Last Refresh: %1" ).arg( sTime ), STATUSBAR_FIELD_LAST_REFRESH );
 }
 
 void KShowMailApp::slotStatusHelpMsg(const QString &text)
@@ -515,20 +509,6 @@ bool KShowMailApp::event(QEvent *e)
   return(KMainWindow::event(e));
 }
 
-void KShowMailApp::slotSetupFilters ()
-{
-  if (m_state != idle)
-  {
-    kapp->beep ();
-    return;
-  }
-  m_state = configure;
-  FilterDialog dlg (this, "Filters");
-  if (dlg.exec () == QDialog::Accepted)
-    m_actionSave->setEnabled (true);
-  m_state = idle;
-}
-
 void KShowMailApp::slotForceClose()
 {
     m_bForceClose = true;
@@ -666,15 +646,6 @@ void KShowMailApp::slotRefreshReady( )
 
   //prepare next refresh cycle
   initNextRefresh();
-
-  //apply filters
-  m_ConfigList.applyFilters();
-
-  //perform automatic delete if desired
-  if (Filter::_status == Filter::automatic)
-  {
-    slotDelete ();
-  }
 }
 
 void KShowMailApp::initFirstRefresh( )
@@ -784,6 +755,9 @@ void KShowMailApp::slotSetup( )
     SetupDialog->addModule( "kshowmailconfiggeneral.desktop" );
     SetupDialog->addModule( "kshowmailconfigdisplay.desktop" );
     SetupDialog->addModule( "kshowmailconfigactions.desktop" );
+    SetupDialog->addModule( "kshowmailconfigfilter.desktop" );
+    SetupDialog->addModule( "kshowmailconfigspamcheck.desktop" );
+    SetupDialog->addModule( "kshowmailconfiglog.desktop" );
 
     //If the configuration was changed, it will call slotConfChanged
     connect( SetupDialog, SIGNAL( configCommitted() ), this, SLOT( slotConfChanged() ) );
@@ -803,5 +777,85 @@ void KShowMailApp::slotConfChanged( )
   //read the new setup
   m_ConfigList.refreshSetup( m_pView->m_pListAccounts );
   m_pView->refreshSetup();
+  fLog.loadSetup();
+
+}
+
+void KShowMailApp::slotShowFilterLog( )
+{
+  //open dialog
+  FilterLogView view( this, &fLog );
+  view.exec();
+
+  refreshFilterStatusBar();
+}
+
+void KShowMailApp::refreshFilterStatusBar( )
+{
+  int numberDeletedLastRefresh = m_ConfigList.numberDeletedMailsLastRefresh();
+  int numberDeletedSinceStart = m_ConfigList.numberDeletedMailsStart();
+  int numberDeletedLog = fLog.numberDeletedMails();
+
+  int numberMovedLastRefresh = m_ConfigList.numberMovedMailsLastRefresh();
+  int numberMovedSinceStart = m_ConfigList.numberMovedMailsStart();
+  int numberMovedLog = fLog.numberMovedMails();
+
+  int numberIgnored = m_ConfigList.numberIgnoredMails();
+
+  statusBar()->changeItem( i18n( "Filter: Deleted: %1/%2/%3; Moved: %4/%5/%6; Ignored: %7" ).arg( numberDeletedLastRefresh ).arg( numberDeletedSinceStart ).arg( numberDeletedLog ).arg( numberMovedLastRefresh ).arg( numberMovedSinceStart ).arg( numberMovedLog ).arg( numberIgnored ), STATUSBAR_FIELD_FILTER );
+}
+
+void KShowMailApp::slotAddToWhitelist( )
+{
+  //get new entries
+  QStringList newEntries = m_ConfigList.getSelectedSenders();
+
+  //get previous whitelist from config
+  KConfig* config = KApplication::kApplication()->config();
+  config->setGroup( CONFIG_GROUP_FILTER );
+  QStringList whiteList = config->readListEntry( CONFIG_ENTRY_FILTER_WHITELIST );
+
+  //append new entries
+  for( QStringList::Iterator it = newEntries.begin(); it != newEntries.end(); ++it )
+  {
+    if( whiteList.contains( *it ) == 0 )
+    {
+      whiteList.append( *it );
+    }
+  }
+
+  //write extended whitelist back
+  config->writeEntry( CONFIG_ENTRY_FILTER_WHITELIST, whiteList );
+  config->sync();
+
+  //load new config
+  m_ConfigList.refreshFilterSetup();
+}
+
+void KShowMailApp::slotAddToBlacklist( )
+{
+  //get new entries
+  QStringList newEntries = m_ConfigList.getSelectedSenders();
+
+  //get previous blacklist from config
+  KConfig* config = KApplication::kApplication()->config();
+  config->setGroup( CONFIG_GROUP_FILTER );
+  QStringList blackList = config->readListEntry( CONFIG_ENTRY_FILTER_BLACKLIST );
+
+  //append new entries
+  for( QStringList::Iterator it = newEntries.begin(); it != newEntries.end(); ++it )
+  {
+    if( blackList.contains( *it ) == 0 )
+    {
+      blackList.append( *it );
+    }
+  }
+
+  //write extended whitelist back
+  config->writeEntry( CONFIG_ENTRY_FILTER_BLACKLIST, blackList );
+  config->sync();
+
+  //load new config
+  m_ConfigList.refreshFilterSetup();
 }
 
