@@ -69,16 +69,6 @@ void ShowRecord::readStoredMails( QDomElement& parent )
   }
 }
 
-void ShowRecord::applyFilters ()
-{
-  //Loop over all mails in this list
-  for( ShowRecordElem* pElem = first(); pElem; pElem = next() )
-  {
-    //apply the filters to the current mail
-    pElem->applyFilters ();
-  }
-}
-
 bool ShowRecord::hasSelectedMails( )
 {
   bool selected = false;    //TRUE when a selected mail was found
@@ -556,3 +546,124 @@ bool ShowRecord::isNew( QString uid ) const
 
   return newMail;
 }
+
+void ShowRecord::applyHeaderFilter( HeaderFilter * filter, QString account, MailNumberList_Type& deleteList, MailToDownloadMap_Type& downloadList, int& nmbIgnoredMails, FilterLog* log )
+{
+
+  MailNumberList_Type mailsToIgnore;  //this list holds the numbers of all mails, which shall be ignored
+
+    //Loop over all mails in this list
+  for( ShowRecordElem* pElem = first(); pElem; pElem = next() )
+  {
+    //apply the filters to the current mail
+    QString mailbox;
+    FilterAction_Type action = pElem->applyHeaderFilter( filter, account, mailbox, log );
+
+    //do recommend action
+    //we don't need to do everything for action MARK, because ShowRecordElem::applyHeaderFilter() marks the mail entry itself
+    struct DownloadActionParams_Type params;
+    switch( action )
+    {
+      case FActDelete     : deleteList.append( pElem->number() ); break;
+      case FActMove       : params.action = FActMove;
+                            params.mailbox = mailbox;
+                            downloadList.insert( pElem->number(), params );
+                            break;
+      case FActIgnore     : mailsToIgnore.append( pElem->number() ); break;
+      case FActSpamcheck  : params.action = FActSpamcheck;
+                            downloadList.insert( pElem->number(), params );
+      default          : break;
+    }
+  }
+
+  //remove all mails which shall be ignored from the mail list
+  nmbIgnoredMails = mailsToIgnore.count();
+  MailNumberList_Type::iterator it;
+  for ( it = mailsToIgnore.begin(); it != mailsToIgnore.end(); ++it )
+    removeMail( *it );
+
+
+}
+
+void ShowRecord::writeToMoveLog( FilterLog * log, int number, QString account, QString mailbox )
+{
+  QPtrListIterator<ShowRecordElem> it( *this );   //iterator for the mail list
+  ShowRecordElem* mail;                           //current mail
+  bool found = false;                             //True, when the wanted mail was found
+
+  //looking for the mail with the number 'number'
+  while( ( mail = it.current() ) != NULL && !found )
+  {
+    //increment iterator to next mail
+    ++it;
+
+    //if the current mail has the given number, set the header
+    if( mail->number() == number )
+    {
+      mail->writeToMoveLog( log, account, mailbox );
+      found = true;
+    }
+  }
+}
+
+void ShowRecord::writeToDeleteLog( FilterLog * log, int number, QString account )
+{
+  QPtrListIterator<ShowRecordElem> it( *this );   //iterator for the mail list
+  ShowRecordElem* mail;                           //current mail
+  bool found = false;                             //True, when the wanted mail was found
+
+  //looking for the mail with the number 'number'
+  while( ( mail = it.current() ) != NULL && !found )
+  {
+    //increment iterator to next mail
+    ++it;
+
+    //if the current mail has the given number, set the header
+    if( mail->number() == number )
+    {
+      mail->writeToDeleteLog( log, account );
+      found = true;
+    }
+  }
+}
+
+void ShowRecord::setMarkAtNextViewRefresh( int number )
+{
+  QPtrListIterator<ShowRecordElem> it( *this );   //iterator for the mail list
+  ShowRecordElem* mail;                           //current mail
+  bool found = false;                             //True, when the wanted mail was found
+
+  //looking for the mail with the number 'number'
+  while( ( mail = it.current() ) != NULL && !found )
+  {
+    //increment iterator to next mail
+    ++it;
+
+    //if the current mail has the given number, set the header
+    if( mail->number() == number )
+    {
+      mail->setMarkAtNextViewRefresh();
+      found = true;
+    }
+  }
+}
+
+QStringList ShowRecord::getSelectedSenders( ) const
+{
+  QStringList senders;                           //contains the senders
+  QPtrListIterator<ShowRecordElem> it( *this );   //iterator for the mail list
+  ShowRecordElem* mail;                           //current mail
+
+  //iterate over all mails
+  while( ( mail = it.current() ) != NULL )
+  {
+    //increment iterator to next mail
+    ++it;
+
+    //if the mail is selected, append subject to list
+    if( mail->isSelected() )
+      senders.append( mail->from() );
+  }
+
+  return senders;
+}
